@@ -29,14 +29,14 @@ readline("Enter for next step")
 
 z<-aws.segment(y+size*y1,level=0,delta=delta,hmax=1000,graph=TRUE)
 
-ergs<-c(sum(z@theta>0&size*y1>delta),sum(z@theta>0&size*y1<=delta),sum(z@theta==0&size*y1>delta),sum(z@theta==0&size*y1<=delta))
+ergs<-c(sum(z@segment>0&size*y1>delta),sum(z@segment>0&size*y1<=delta),sum(z@segment<=0&size*y1>delta),sum(z@segment<=0&size*y1<=delta))
 names(ergs) <-c("Correct alternative","False positive","False negative","Correct hypothesis")
 print(ergs)
 
 readline("Enter for next step")
 
 plot(size*y1>delta,col=1,type="l")
-lines(z@theta,col=2)
+lines(z@segment,col=2)
 title("True segments (black) and segmentation results (red)" )
 
 if(! readline("keep files and device (N/Y) :") %in% c("y","Y")){ 
